// COMLogXML.cpp : Implementation of CCOMLogXMLXML
#include "stdafx.h"
#include "LogBook.h"
#include "COMLogXML.h"
#include "ComLogbook.h"

/////////////////////////////////////////////////////////////////////////////
// CCOMLogXMLXML
/////////////////////////////////////////////////////////////////////////////
// CCOMLogXML
   
const CString CR = "\n\r";
const CString FOOTER = "</Logbook>";

CCOMLogXML::CCOMLogXML()
{
   m_dwMethodIndex = 0;
   m_dwEventIndex = 0;
   m_dwErrorIndex = 0;
}

CCOMLogXML::~CCOMLogXML()
{
}

HRESULT  CCOMLogXML::FinalConstruct(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   return S_OK;
}

void CCOMLogXML::FinalRelease(void)
{
   AFX_MANAGE_STATE(AfxGetStaticModuleState())

   if (m_LogFile.m_pStream)
   {
      TRY
      {
         m_LogFile.Flush();
         m_LogFile.Close();   
      }
      CATCH(CFileException, e)
      {
         ATLASSERT(FALSE);
      }
      END_CATCH
   }
}

/////////////////////////////////////////////////////////////////////////////
// IObjectConstruct
STDMETHODIMP CCOMLogXML::Construct(IDispatch * pCtorObj)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
   HRESULT hres = S_OK;
	IObjectConstructString* pString = NULL;
   hres = pCtorObj->QueryInterface(IID_IObjectConstructString, (void **)&pString);
   ASSERT_AND_RETURN(hres);

	BSTR	bstrConstruct;
	hres = pString->get_ConstructString(&bstrConstruct);
   pString->Release();
   ASSERT_AND_RETURN(hres);

   CString sPath = bstrConstruct;
   SysFreeString(bstrConstruct);

   BOOL bres = TRUE;

   if (sPath.IsEmpty())
   {

      CString sLogName;
      bres = sLogName.LoadString(IDS_LOG_FILENAME);
      if (FALSE == bres)
      {
         sLogName = "Logbook.xml";
      }

      TCHAR lpFileName[MAX_PATH];
      DWORD dwLength = ::GetModuleFileName(AfxGetInstanceHandle(), lpFileName, MAX_PATH - 2);
      if (dwLength)
      {
         sPath = lpFileName;
         CString sFileName = CEventLogger::FindPathInFileName(lpFileName);
         int nFileNameIndex = sPath.Find(sFileName);
         sPath.Delete(nFileNameIndex, sFileName.GetLength());
      }

      sPath += sLogName;
   }

   CFileException fileError;
   bres = m_LogFile.Open(sPath, 
                         CFile::shareDenyNone | CFile::modeCreate |
                         CFile::modeWrite     | CFile::typeText, &fileError); 
   if (FALSE == bres)
   {
      return E_FAIL;
   }

   TRY
   {
      m_LogFile.Seek(0, CFile::end);
   }
   CATCH(CFileException, e)
   {
      return E_FAIL;
   }
   END_CATCH

   return FormatNewFile();
}
////////////////////////////////////////////////////////////
//XML formating methods
HRESULT CCOMLogXML::FormatNewFile()
{
   HRESULT hres = S_OK;

   CString sHeader = "<?xml version=\"1.0\"?>"+ CR+"<Logbook>"+CR;  
   
   hres = WriteString(sHeader);
   ASSERT_AND_RETURN(hres);
 
   hres = AddFooter();
   ASSERT_AND_RETURN(hres);

   return S_OK;

}
HRESULT CCOMLogXML::SetPointer()
{
   const LONG OFFSET = FOOTER.GetLength()+3;
   TRY
   {
      m_LogFile.Seek(-OFFSET, CFile::current);
   }
   CATCH(CFileException, e)
   {
      return E_FAIL;
   }
   END_CATCH
   
   return S_OK;
}

HRESULT CCOMLogXML::AddFooter()
{
   HRESULT hres = S_OK;
   hres = WriteString(FOOTER);
   ASSERT_AND_RETURN(hres);

   hres = WriteString(CR);
   ASSERT_AND_RETURN(hres);

   return hres;
}

//ILogbook implementation
STDMETHODIMP CCOMLogXML::LogMethod(LOG_ENTRY *pMethodEntry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   HRESULT hres = S_OK;

   hres = SetPointer();

   ASSERT_AND_RETURN(hres);

   CString sEntry;
   CString sEntryBody;
   CString sLocation;
   CString sID;
   CString sTime;

   GetIDItem(pMethodEntry,sID);
   GetLocationItem(pMethodEntry,sLocation);
   GetTimeItem(pMethodEntry,sTime);

   sEntryBody = sID + sLocation+ sTime;

   sEntry.Format(
   "<MethodEntry>#%u\n"
   "%s\n"
   "</MethodEntry>\n"
   "%s\n",
   ++m_dwMethodIndex,sEntryBody,FOOTER);

   hres = WriteString(sEntry);
   ASSERT_AND_RETURN(hres);

   return S_OK;
}

STDMETHODIMP CCOMLogXML::LogError(LOG_ENTRY *pErrorEntry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   HRESULT hres = S_OK;

   hres = SetPointer();

   ASSERT_AND_RETURN(hres);

   CString sEntry;
   CString sEntryBody;
   CString sLocation;
   CString sID;
   CString sTime;
   CString sError;

   GetErrorItem(pErrorEntry,sError);
   GetIDItem(pErrorEntry,sID);
   GetLocationItem(pErrorEntry,sLocation);
   GetTimeItem(pErrorEntry,sTime);

   sEntryBody = sID + sLocation+ sTime + sError;

   sEntry.Format(
   "<ErrorEntry>#%u\n"
   "%s\n"
   "</ErrorEntry>\n"
   "%s\n",
   ++m_dwErrorIndex,sEntryBody,FOOTER);

   hres = WriteString(sEntry);
   ASSERT_AND_RETURN(hres);

   return S_OK;
}


STDMETHODIMP CCOMLogXML::LogEvent(LOG_ENTRY *pEventEntry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   HRESULT hres = S_OK;

   hres = SetPointer();

   ASSERT_AND_RETURN(hres);

   CString sEntry;
   CString sEntryBody;
   CString sID;
   CString sTime;
   CString sEvent;

   GetEventItem(pEventEntry,sEvent);
   GetIDItem(pEventEntry,sID);
   GetTimeItem(pEventEntry,sTime);

   sEntryBody = sID + sTime + sEvent;

   sEntry.Format(
   "<EventEntry>#%u\n"
   "%s\n"
   "</EventEntry>\n"
   "%s\n",
   ++m_dwEventIndex,sEntryBody,FOOTER);

   hres = WriteString(sEntry);
   ASSERT_AND_RETURN(hres);

   return S_OK;
}
/////////////////////////////////////////////////////////////
//Helper methods
void CCOMLogXML::GetEventItem(const LOG_ENTRY* pEvent,CString& sEvent)const
{
   sEvent = "";

   sEvent.Format(
      "<EventDescription>\n"
      "   %s\n"
      "</EventDescription>\n",
      CString(pEvent->bstrDescription));
}

void CCOMLogXML::GetErrorItem(const LOG_ENTRY* pEvent,CString& sError)const
{
   sError = "";

   _com_error comErr(pEvent->hres);
   CString sHres = comErr.ErrorMessage();
   
   CString sIID("IID_NULL");
   ConvertGUID(pEvent->iidError,sIID);

   sError.Format(
      "<ErrorInfo>\n"
      "   <ReturnedValue>\n"
      "      %s\n"
      "   </ReturnedValue>\n"
      "   <ErrorCode>\n"
      "      %4d\n"
      "   </ErrorCode>\n"
      "   <IID>\n"
      "      %s\n"
      "   </IID>\n"
      "   <ErrorDescription>\n"
      "      %s\n"
      "   </ErrorDescription>\n"
      "</ErrorInfo>\n"
      ,sHres,pEvent->dwErrorCode,sIID,CString(pEvent->bstrDescription));
}

void CCOMLogXML::GetTimeItem(const LOG_ENTRY* pEvent,CString& sTime)const
{
   SYSTEMTIME systemTime;
   GetTime(pEvent,systemTime);


   sTime = "";
   sTime.Format(
      "<Time>\n"
      "   <Date>\n"
      "      %02d/%02d/%04d\n"
      "   </Date>\n"
      "   <Hour>\n"
      "      %02d:%02d:%02d\n"
      "   </Hour>\n"
      "</Time>\n",
      systemTime.wMonth,systemTime.wDay,systemTime.wYear,systemTime.wHour,systemTime.wMinute,systemTime.wSecond);
}

void CCOMLogXML::GetLocationItem(const LOG_ENTRY* pEvent,CString& sLocation)const
{
   sLocation = "";
   sLocation.Format(
      "<Location>\n"
      "   <MachineName>\n"
      "      %s\n"
      "   </MachineName>\n"
      "   <SourceFile>\n"
      "      %s\n"
      "   </SourceFile>\n"
      "   <ModuleName>\n"
      "      %s\n"
      "   </ModuleName>\n"
      "   <MethodName>\n"
      "      %s\n"
      "   </MethodName>\n"
      "   <LineNumber>\n"
      "      %u\n"
      "   </LineNumber>\n"
      "</Location>\n",
      CString(pEvent->bstrMachineName),CString(pEvent->bstrSourceFileName),CString(pEvent->bstrModuleName),CString(pEvent->bstrMethodName),pEvent->dwLineNumber);
}


void CCOMLogXML::GetIDItem(const LOG_ENTRY* pEvent,CString& sID)const
{
   sID = "";
   
   CString sActivityID("No Activity");
   CString sTransactionID("No Transaction");
   CString sContextID("Default Context");

   ConvertGUID(pEvent->guidActivityID,sActivityID);
   ConvertGUID(pEvent->guidTransactionID,sTransactionID);
   ConvertGUID(pEvent->guidContextID,sContextID);

   sID.Format(     
      "<ID>\n"
      "   <ProcessID>\n"
      "      %u\n"
      "   </ProcessID>\n"
      "   <ThreadID>\n"
      "      %04X\n"
      "   </ThreadID>\n"
      "   <ContextID>\n"
      "      %s\n"
      "   </ContextID>\n"
      "   <AcrtivityID>\n"
      "      %s\n"
      "   </AcrtivityID>\n"
      "   <TransactionID>\n"
      "      %s\n"
      "   </TransactionID>\n"
      "</ID>\n",pEvent->dwProcessID,pEvent->dwThreadID,sContextID,sActivityID,sTransactionID);
}



void CCOMLogXML::GetTime(const LOG_ENTRY* pEvent, SYSTEMTIME& systemTime) const
{
   ::ZeroMemory(&systemTime, sizeof(systemTime));

   FILETIME localTime;
   BOOL bres = ::FileTimeToLocalFileTime(&pEvent->eventTime, &localTime);
   if (bres)
   {
      bres = ::FileTimeToSystemTime(&localTime, &systemTime);
   }
}

HRESULT CCOMLogXML::WriteString(const CString& sString)
{
   HRESULT hres = S_OK;

   if (m_LogFile.m_pStream)
   {
      TRY
      {
         m_LogFile.WriteString(sString);
         m_LogFile.Flush();   
      }
      CATCH(CFileException, e)
      {
         hres = E_FAIL;
      }
      END_CATCH
   }

   return hres;
}

void CCOMLogXML::ConvertGUID(REFGUID guidConst,CString& sGUID)const
{
   if (GUID_NULL == guidConst)
   {
      return;
   }
   else
   {
      USES_CONVERSION;
      const int nSize = 64;
      WCHAR wszID[nSize];
      int nNumChars = ::StringFromGUID2(guidConst, wszID, nSize);
      VERIFY(nNumChars);
      sGUID = W2A(wszID);
   }
}

